// src/floating-panel.anatomy.ts
import { createAnatomy } from "@zag-js/anatomy";
var anatomy = createAnatomy("floating-panel").parts(
  "trigger",
  "positioner",
  "content",
  "header",
  "body",
  "title",
  "resizeTrigger",
  "dragTrigger",
  "minimizeTrigger",
  "maximizeTrigger",
  "closeTrigger",
  "restoreTrigger",
  "dock"
);
var parts = anatomy.build();

// src/floating-panel.connect.ts
import { getEventKey, getEventStep, getNativeEvent } from "@zag-js/dom-event";
import { dataAttr, getEventTarget, isSelfEvent } from "@zag-js/dom-query";

// src/floating-panel.dom.ts
import { createScope } from "@zag-js/dom-query";
var dom = createScope({
  getTriggerId: (ctx) => `float:${ctx.id}:trigger`,
  getPositionerId: (ctx) => `float:${ctx.id}:positioner`,
  getContentId: (ctx) => `float:${ctx.id}:content`,
  getTitleId: (ctx) => `float:${ctx.id}:title`,
  getHeaderId: (ctx) => `float:${ctx.id}:header`,
  getTriggerEl: (ctx) => dom.getById(ctx, dom.getTriggerId(ctx)),
  getPositionerEl: (ctx) => dom.getById(ctx, dom.getPositionerId(ctx)),
  getContentEl: (ctx) => dom.getById(ctx, dom.getContentId(ctx)),
  getHeaderEl: (ctx) => dom.getById(ctx, dom.getHeaderId(ctx))
});

// src/get-resize-axis-style.ts
function getResizeAxisStyle(axis) {
  switch (axis) {
    case "n":
      return {
        cursor: "n-resize",
        width: "100%",
        left: "50%",
        translate: "-50%"
      };
    case "e":
      return {
        cursor: "e-resize",
        height: "100%",
        right: 0,
        top: "50%",
        translate: "0 -50%"
      };
    case "s":
      return {
        cursor: "s-resize",
        width: "100%",
        bottom: 0,
        left: "50%",
        translate: "-50%"
      };
    case "w":
      return {
        cursor: "w-resize",
        height: "100%",
        left: 0,
        top: "50%",
        translate: "0 -50%"
      };
    case "se":
      return {
        cursor: "se-resize",
        bottom: 0,
        right: 0
      };
    case "sw":
      return {
        cursor: "sw-resize",
        bottom: 0,
        left: 0
      };
    case "ne":
      return {
        cursor: "ne-resize",
        top: 0,
        right: 0
      };
    case "nw":
      return {
        cursor: "nw-resize",
        top: 0,
        left: 0
      };
    default:
      throw new Error(`Invalid axis: ${axis}`);
  }
}

// src/floating-panel.connect.ts
function connect(state, send, normalize) {
  const isOpen = state.hasTag("open");
  const isDragging = state.matches("open.dragging");
  const isResizing = state.matches("open.resizing");
  return {
    isOpen,
    isDragging,
    isResizing,
    triggerProps: normalize.button({
      ...parts.trigger.attrs,
      type: "button",
      disabled: state.context.isDisabled,
      id: dom.getTriggerId(state.context),
      "data-state": isOpen ? "open" : "closed",
      "data-dragging": dataAttr(isDragging),
      "aria-controls": dom.getContentId(state.context),
      onClick() {
        if (state.context.isDisabled)
          return;
        send({ type: "OPEN" });
      }
    }),
    positionerProps: normalize.element({
      ...parts.positioner.attrs,
      id: dom.getPositionerId(state.context),
      style: {
        position: "absolute",
        top: "var(--y)",
        left: "var(--x)"
      }
    }),
    contentProps: normalize.element({
      ...parts.content.attrs,
      role: "dialog",
      tabIndex: 0,
      hidden: !isOpen,
      id: dom.getContentId(state.context),
      "aria-labelledby": dom.getTitleId(state.context),
      "data-state": isOpen ? "open" : "closed",
      "data-dragging": dataAttr(isDragging),
      style: {
        position: "relative",
        width: "var(--width)",
        height: "var(--height)",
        overflow: state.context.isMinimized ? "hidden" : void 0
      },
      onKeyDown(event) {
        if (!isSelfEvent(getNativeEvent(event)))
          return;
        const step = getEventStep(event) * state.context.gridSize;
        const keyMap = {
          Escape() {
            send("ESCAPE");
          },
          ArrowLeft() {
            send({ type: "MOVE", direction: "left", step });
          },
          ArrowRight() {
            send({ type: "MOVE", direction: "right", step });
          },
          ArrowUp() {
            send({ type: "MOVE", direction: "up", step });
          },
          ArrowDown() {
            send({ type: "MOVE", direction: "down", step });
          }
        };
        const handler = keyMap[getEventKey(event, state.context)];
        if (handler) {
          event.preventDefault();
          handler(event);
        }
      }
    }),
    closeTriggerProps: normalize.button({
      ...parts.closeTrigger.attrs,
      disabled: state.context.isDisabled,
      "aria-label": "Close Window",
      type: "button",
      onClick() {
        send("CLOSE");
      }
    }),
    minimizeTriggerProps: normalize.button({
      ...parts.minimizeTrigger.attrs,
      disabled: state.context.isDisabled,
      "aria-label": "Minimize Window",
      hidden: state.context.isStaged,
      type: "button",
      onClick() {
        send("MINIMIZE");
      }
    }),
    maximizeTriggerProps: normalize.button({
      ...parts.maximizeTrigger.attrs,
      disabled: state.context.isDisabled,
      "aria-label": "Maximize Window",
      hidden: state.context.isStaged,
      type: "button",
      onClick() {
        send("MAXIMIZE");
      }
    }),
    restoreTriggerProps: normalize.button({
      ...parts.restoreTrigger.attrs,
      disabled: state.context.isDisabled,
      "aria-label": "Restore Window",
      hidden: !state.context.isStaged,
      type: "button",
      onClick() {
        send("RESTORE");
      }
    }),
    getResizeTriggerProps(props2) {
      return normalize.element({
        ...parts.resizeTrigger.attrs,
        "data-disabled": dataAttr(!state.context.canResize),
        "data-axis": props2.axis,
        onPointerDown(event) {
          if (!state.context.canResize || event.button == 2)
            return;
          event.currentTarget.setPointerCapture(event.pointerId);
          event.stopPropagation();
          send({
            type: "RESIZE_START",
            axis: props2.axis,
            position: { x: event.clientX, y: event.clientY }
          });
        },
        onPointerUp(event) {
          if (!state.context.canResize)
            return;
          const node = event.currentTarget;
          if (node.hasPointerCapture(event.pointerId)) {
            node.releasePointerCapture(event.pointerId);
          }
        },
        style: {
          position: "absolute",
          touchAction: "none",
          ...getResizeAxisStyle(props2.axis)
        }
      });
    },
    dragTriggerProps: normalize.element({
      ...parts.dragTrigger.attrs,
      "data-disabled": dataAttr(!state.context.canDrag),
      onPointerDown(event) {
        if (!state.context.canDrag || event.button == 2)
          return;
        const target = getEventTarget(getNativeEvent(event));
        if (target?.closest("button")) {
          return;
        }
        event.currentTarget.setPointerCapture(event.pointerId);
        event.stopPropagation();
        send({
          type: "DRAG_START",
          pointerId: event.pointerId,
          position: { x: event.clientX, y: event.clientY }
        });
      },
      onPointerUp(event) {
        if (!state.context.canDrag)
          return;
        const node = event.currentTarget;
        if (node.hasPointerCapture(event.pointerId)) {
          node.releasePointerCapture(event.pointerId);
        }
      },
      onDoubleClick() {
        send(state.context.isMaximized ? "RESTORE" : "MAXIMIZE");
      },
      style: {
        userSelect: "none",
        touchAction: "none",
        cursor: "move"
      }
    }),
    titleProps: normalize.element({
      ...parts.title.attrs,
      id: dom.getTitleId(state.context)
    }),
    headerProps: normalize.element({
      ...parts.header.attrs,
      id: dom.getHeaderId(state.context),
      "data-dragging": dataAttr(isDragging)
    }),
    bodyProps: normalize.element({
      ...parts.body.attrs,
      "data-dragging": dataAttr(isDragging),
      hidden: state.context.isMinimized
    })
  };
}

// src/floating-panel.machine.ts
import { createMachine, guards } from "@zag-js/core";
import { addDomEvent, trackPointerMove } from "@zag-js/dom-event";
import { isHTMLElement } from "@zag-js/dom-query";
import {
  addPoints,
  clampPoint,
  clampSize,
  constrainRect,
  createRect,
  getElementRect,
  getWindowRect,
  isPointEqual,
  isSizeEqual,
  resizeRect,
  subtractPoints
} from "@zag-js/rect-utils";
import { compact, invariant, isEqual, match, pick } from "@zag-js/utils";
var { not } = guards;
function machine(userContext) {
  const ctx = compact(userContext);
  return createMachine(
    {
      id: "floating-panel",
      initial: ctx.open ? "open" : "closed",
      context: {
        size: { width: 320, height: 400 },
        position: { x: 300, y: 100 },
        gridSize: 1,
        disabled: false,
        resizable: true,
        draggable: true,
        ...ctx,
        stage: void 0,
        lastEventPosition: null,
        prevPosition: null,
        prevSize: null,
        boundaryRect: null
      },
      computed: {
        isMaximized: (ctx2) => ctx2.stage === "maximized",
        isMinimized: (ctx2) => ctx2.stage === "minimized",
        isStaged: (ctx2) => !!ctx2.stage,
        isDisabled: (ctx2) => !!ctx2.disabled,
        canResize: (ctx2) => (ctx2.resizable || !ctx2.isDisabled) && !ctx2.stage,
        canDrag: (ctx2) => (ctx2.draggable || !ctx2.isDisabled) && !ctx2.isMaximized
      },
      watch: {
        position: ["setPositionStyle"],
        size: ["setSizeStyle"]
      },
      states: {
        closed: {
          tags: ["closed"],
          on: {
            OPEN: {
              target: "open",
              actions: ["invokeOnOpen", "setPositionStyle", "setSizeStyle"]
            }
          }
        },
        open: {
          tags: ["open"],
          activities: ["trackBoundaryRect"],
          on: {
            DRAG_START: {
              guard: not("isMaximized"),
              target: "open.dragging",
              actions: ["setPrevPosition"]
            },
            RESIZE_START: {
              guard: not("isMinimized"),
              target: "open.resizing",
              actions: ["setPrevSize"]
            },
            CLOSE: {
              target: "closed",
              actions: ["invokeOnClose", "resetRect"]
            },
            ESCAPE: {
              guard: "closeOnEsc",
              target: "closed",
              actions: ["invokeOnClose", "resetRect"]
            },
            MINIMIZE: {
              actions: ["setMinimized", "invokeOnMinimize"]
            },
            MAXIMIZE: {
              actions: ["setMaximized", "invokeOnMaximize"]
            },
            RESTORE: {
              actions: ["setRestored"]
            },
            MOVE: {
              actions: ["setPositionFromKeybord"]
            }
          }
        },
        "open.dragging": {
          tags: ["open"],
          activities: ["trackPointerMove", "trackBoundaryRect"],
          exit: ["clearPrevPosition"],
          on: {
            DRAG: {
              actions: ["setPosition"]
            },
            DRAG_END: {
              target: "open",
              actions: ["invokeOnDragEnd"]
            },
            CLOSE: {
              target: "closed",
              actions: ["invokeOnClose", "resetRect"]
            },
            ESCAPE: {
              target: "open"
            }
          }
        },
        "open.resizing": {
          tags: ["open"],
          activities: ["trackPointerMove", "trackBoundaryRect"],
          exit: ["clearPrevSize"],
          on: {
            DRAG: {
              actions: ["setSize"]
            },
            DRAG_END: {
              target: "open",
              actions: ["invokeOnResizeEnd"]
            },
            CLOSE: {
              target: "closed",
              actions: ["invokeOnClose", "resetRect"]
            },
            ESCAPE: {
              target: "open"
            }
          }
        }
      }
    },
    {
      guards: {
        closeOnEsc: (ctx2) => !!ctx2.closeOnEscape,
        isMaximized: (ctx2) => ctx2.isMaximized,
        isMinimized: (ctx2) => ctx2.isMinimized
      },
      activities: {
        trackPointerMove(ctx2, _evt, { send }) {
          const doc = dom.getDoc(ctx2);
          return trackPointerMove(doc, {
            onPointerMove({ point, event }) {
              const { altKey, shiftKey } = event;
              send({ type: "DRAG", position: point, axis: _evt.axis, altKey, shiftKey });
            },
            onPointerUp() {
              send("DRAG_END");
            }
          });
        },
        trackBoundaryRect(ctx2) {
          const win = dom.getWin(ctx2);
          const el = ctx2.getBoundaryEl?.();
          const adjust = (boundary) => {
            const boundaryRect = pick(boundary, ["width", "height", "x", "y"]);
            ctx2.boundaryRect = boundaryRect;
            const res = ctx2.isMaximized ? boundaryRect : constrainRect(
              {
                ...ctx2.position,
                ...ctx2.size
              },
              boundaryRect
            );
            set.size(ctx2, pick(res, ["width", "height"]));
            set.position(ctx2, pick(res, ["x", "y"]));
          };
          if (isHTMLElement(el)) {
            const exec2 = () => adjust(getElementRect(el));
            exec2();
            const obs = new win.ResizeObserver(exec2);
            obs.observe(el);
            return () => obs.disconnect();
          }
          const exec = () => adjust(getWindowRect(win));
          exec();
          return addDomEvent(win, "resize", exec);
        }
      },
      actions: {
        setPrevPosition(ctx2, evt) {
          ctx2.prevPosition = { ...ctx2.position };
          ctx2.lastEventPosition = evt.position;
        },
        clearPrevPosition(ctx2) {
          if (!ctx2.preserveOnClose)
            ctx2.prevPosition = null;
          ctx2.lastEventPosition = null;
        },
        setPosition(ctx2, evt) {
          let diff = subtractPoints(evt.position, ctx2.lastEventPosition);
          diff.x = Math.round(diff.x / ctx2.gridSize) * ctx2.gridSize;
          diff.y = Math.round(diff.y / ctx2.gridSize) * ctx2.gridSize;
          let position = addPoints(ctx2.prevPosition, diff);
          position = clampPoint(position, ctx2.size, ctx2.boundaryRect);
          set.position(ctx2, position);
        },
        setPositionStyle(ctx2) {
          const el = dom.getPositionerEl(ctx2);
          el?.style.setProperty("--x", `${ctx2.position.x}px`);
          el?.style.setProperty("--y", `${ctx2.position.y}px`);
        },
        resetRect(ctx2, _evt, { initialContext }) {
          ctx2.stage = void 0;
          if (!ctx2.preserveOnClose) {
            set.position(ctx2, initialContext.position);
            set.size(ctx2, initialContext.size);
          }
        },
        setPrevSize(ctx2, evt) {
          ctx2.prevSize = { ...ctx2.size };
          ctx2.prevPosition = { ...ctx2.position };
          ctx2.lastEventPosition = evt.position;
        },
        clearPrevSize(ctx2) {
          ctx2.prevSize = null;
          ctx2.prevPosition = null;
          ctx2.lastEventPosition = null;
        },
        setSize(ctx2, evt) {
          if (!ctx2.prevSize || !ctx2.prevPosition || !ctx2.lastEventPosition)
            return;
          const prevRect = createRect({ ...ctx2.prevPosition, ...ctx2.prevSize });
          const offset = subtractPoints(evt.position, ctx2.lastEventPosition);
          const nextRect = resizeRect(prevRect, offset, evt.axis, {
            scalingOriginMode: evt.altKey ? "center" : "extent",
            lockAspectRatio: !!ctx2.lockAspectRatio || evt.shiftKey
          });
          let nextSize = pick(nextRect, ["width", "height"]);
          let nextPosition = pick(nextRect, ["x", "y"]);
          nextSize = clampSize(nextSize, ctx2.minSize, ctx2.maxSize);
          set.size(ctx2, nextSize);
          if (nextPosition) {
            const point = clampPoint(nextPosition, nextSize, ctx2.boundaryRect);
            set.position(ctx2, point);
          }
        },
        setSizeStyle(ctx2) {
          const el = dom.getPositionerEl(ctx2);
          el?.style.setProperty("--width", `${ctx2.size.width}px`);
          el?.style.setProperty("--height", `${ctx2.size.height}px`);
        },
        setMaximized(ctx2) {
          set.stage(ctx2, "maximized");
          ctx2.prevSize = ctx2.size;
          ctx2.prevPosition = ctx2.position;
          set.position(ctx2, { x: 0, y: 0 });
          set.size(ctx2, pick(ctx2.boundaryRect, ["height", "width"]));
        },
        setMinimized(ctx2) {
          set.stage(ctx2, "minimized");
          ctx2.prevSize = ctx2.size;
          ctx2.prevPosition = ctx2.position;
          const headerEl = dom.getHeaderEl(ctx2);
          if (!headerEl)
            return;
          const size = {
            ...ctx2.size,
            height: headerEl?.offsetHeight
          };
          set.size(ctx2, size);
        },
        setRestored(ctx2) {
          set.stage(ctx2, void 0);
          if (ctx2.prevSize) {
            set.size(ctx2, ctx2.prevSize);
            ctx2.prevSize = null;
          }
          if (ctx2.prevPosition) {
            set.position(ctx2, ctx2.prevPosition);
            ctx2.prevPosition = null;
          }
        },
        setPositionFromKeybord(ctx2, evt) {
          invariant(evt.step == null, "step is required");
          let nextPosition = match(evt.direction, {
            left: { x: ctx2.position.x - evt.step, y: ctx2.position.y },
            right: { x: ctx2.position.x + evt.step, y: ctx2.position.y },
            up: { x: ctx2.position.x, y: ctx2.position.y - evt.step },
            down: { x: ctx2.position.x, y: ctx2.position.y + evt.step }
          });
          nextPosition = clampPoint(nextPosition, ctx2.size, ctx2.boundaryRect);
          set.position(ctx2, nextPosition);
        },
        invokeOnOpen(ctx2) {
          ctx2.onOpenChange?.({ open: true });
        },
        invokeOnClose(ctx2) {
          ctx2.onOpenChange?.({ open: false });
        },
        invokeOnDragEnd(ctx2) {
          ctx2.onPositionChangeEnd?.({ position: ctx2.position });
        },
        invokeOnResizeEnd(ctx2) {
          ctx2.onSizeChangeEnd?.({ size: ctx2.size });
        },
        invokeOnMinimize(ctx2) {
          ctx2.onStageChange?.({ stage: "minimized" });
        },
        invokeOnMaximize(ctx2) {
          ctx2.onStageChange?.({ stage: "maximized" });
        }
      }
    }
  );
}
var set = {
  size(ctx, value) {
    if (isSizeEqual(ctx.size, value))
      return;
    ctx.size = value;
    ctx.onSizeChange?.({ size: value });
  },
  position(ctx, value) {
    if (isPointEqual(ctx.position, value))
      return;
    ctx.position = value;
    ctx.onPositionChange?.({ position: value });
  },
  stage(ctx, value) {
    if (isEqual(ctx.stage, value))
      return;
    ctx.stage = value;
    ctx.onStageChange?.({ stage: value });
  }
};

// src/floating-panel.props.ts
import { createProps } from "@zag-js/types";
import { createSplitProps } from "@zag-js/utils";
var props = createProps()([
  "closeOnEscape",
  "dir",
  "disabled",
  "draggable",
  "getBoundaryEl",
  "getRootNode",
  "gridSize",
  "id",
  "lockAspectRatio",
  "maxSize",
  "minSize",
  "onPositionChange",
  "onPositionChangeEnd",
  "onOpenChange",
  "onStageChange",
  "onSizeChange",
  "onSizeChangeEnd",
  "open",
  "position",
  "preserveOnClose",
  "resizable",
  "size"
]);
var splitProps = createSplitProps(props);
export {
  anatomy,
  connect,
  machine,
  props,
  splitProps
};
//# sourceMappingURL=index.mjs.map